/* 
 * This is simole test only program.
 * This program is distributed  WITHOUT ANY WARRANTY.
 * You can use it on your own risk.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#include <mysql.h>
#include <getopt.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>

int help() {
  execl("/bin/cat", "cat", "usage", 0);
}

main(int argc, char **argv) {
  MYSQL *dbh = (MYSQL *)malloc(sizeof(MYSQL));
  int c;
  int option_index = 0;
  static struct option long_options[] = {
    {"hostname", 1, 0, 'H'},
    {"port",     1, 0, 'p'},
    {"username", 1, 0, 'U'},
    {"password", 1, 0, 'P'},
    {"database", 1, 0, 'D'},
    {"socket",   1, 0, 'S'},
    {"query", 1, 0, 'Q'},
    {"count", 1, 0, 'c'},
    {"verbose", 0, 0, 'v'},
    {"one-connection", 0, 0, 'o'},
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}
  };
  char *hostname=0,
       *username=0,
       *password=0,
       *database=0,
       *socket=0,
       *query=0;

  int  i, j,
       one=0,
       port=3306,
       counter=1000,
       verbose=0;


  while ((c = getopt_long (argc, argv, "H:U:P:D:Q:S:c:voh", long_options, &option_index)) != -1) {
    switch(c) {
      case '?': exit(1); break;
      case 'H': hostname = (char *)strdup(optarg); break;
      case 'p': port     = atoi(optarg); break;
      case 'U': username = (char *)strdup(optarg); break;
      case 'P': password = (char *)strdup(optarg); break;
      case 'D': database = (char *)strdup(optarg); break;
      case 'S': socket   = (char *)strdup(optarg); break;
      case 'Q': query    = (char *)strdup(optarg); break;
      case 'o': one      = 1; break;
      case 'c': counter  = atoi(optarg); break;
      case 'v': verbose  = 1; break;
      case 'h': help(); exit(0);
    }
  }

  if (!((hostname || socket) && query)) {
    help();
    exit(1);
  }

  fprintf(stderr, "**** TEST SQL ****\n  Hostname = %s\n  Username = %s\n  Password = %s\n  Database = %s\n  Query    = %s\n  Mode     = %s\n  Counter  = %i\n******************\n", hostname, username, password, database, query, (one?"One connection":"One query per connection"), counter);

  mysql_init(dbh);

  /* Inicjalizacja polaczenia do bazy danych w trybie one connection */
    if(one) {
      if (!mysql_real_connect(dbh,
			      hostname,
			      username,
			      password,
			      database,
			      port,
			      socket,
			      0)
	 ) {
        fprintf(stderr, "Error: Unable to connect to database. I'd better die.\n");
	exit(1);
      }
    }
  /*******************************************************************/

    for (i=0; i<counter; i++) {
      if (!one) {
        if (!mysql_real_connect(dbh,
        		      hostname,
        		      username,
        		      password,
        		      database,
        		      port,
        		      socket,
        		      0)
        ) {
           printf("i=%i; Error: Unable to connect to database. I'd better die.\n", i);
           exit(1);
	}
      }

      mysql_real_query(dbh, query, strlen(query));
      MYSQL_RES *result;
      result = mysql_store_result(dbh);
      if(verbose) {
	MYSQL_ROW row;
	int num_fields = mysql_num_fields(result);
        printf("i=%i: ", i);
	while(row = mysql_fetch_row(result)) {
	  unsigned long *lengths = mysql_fetch_lengths(result);
	  for(j = 0; j < num_fields; j++) {
	    printf("[%.*s] ", (int) lengths[j], row[j] ? row[j] : "NULL");
	  } printf("\n");
	}
      }
      if (!one) {
	mysql_close(dbh);
      }
    }
  /* Zamykanie polaczenia do bazy danych w trybie one connection */
    if(one) {
      mysql_close(dbh);
    }
  /***************************************************************/
  return 0;
}
